//
// d m a p c m . c
//
// DMA-driven PCM player on GBA
//
// July 20, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "screen.h"
#include "console.h"
#include "printf.h"

// External global symbols

extern char wave_data[];
extern char wave_comment[];
extern int  wave_freq;
extern int  wave_size;

int  wavecnt = 0;			// Wave data counter

// Interrupt handler

void int_handler(void) {
  int flag;

  register(INT_IME) = INT_IME_OFF;	// Disable INTs
  flag = register(INT_IF);		// Read INT flag bits
  if (flag & INT_KEY) {			// This is keypad INT
    register(DMA_CTRL1)   = 0;		// Disable DMA
    register(TMR_CTRL0)   = 0;		// Disable timer
    register4(DMA_DSTAD1) = SND_DAFIFO;	// Reset DMA controller
    register4(DMA_SRCAD1) = (int) wave_data;
    register(DMA_CTRL1)   = DMA_ENABLE + DMA_INTREN + DMA_STARTFIFO + \
			    DMA_SIZE32 + DMA_REPEAT + DMA_SRCINC + DMA_DSTFIX;
    wavecnt = wave_size;		// Reset counter
    register(TMR_CTRL0) =		// Activate timer
      TMR_ENABLE + TMR_1CLOCK;
   }
  if (flag & INT_DMA1) {		// This is DMA1 INT
    wavecnt -= 16;			// Decrement counter
    if (wavecnt <= 0) {			// End of job?
      register(TMR_CTRL0) = 0;		// Then disable timer 0
      register(DMA_CTRL1) = 0;		// and DMA 1
     }
   }
  register(INT_IF) = flag;		// Reset INT flags
  register(INT_IME) = INT_IME_ON;	// Enable INTs
 }

int main() {
  int cnt;

  // Initialize screen

  init_screen();

  // Initialize sound controller

  register(SND_MASTER)  = SND_ENABLE;
  register(SND_DOUTPUT) = SND_DARESET + SND_DAFULL +
                          SND_DALEFT + SND_DARIGHT;

  // Initialize DMA controller

  register4(DMA_SRCAD1) = (int) wave_data;
  register4(DMA_DSTAD1) = SND_DAFIFO;
  register(DMA_CTRL1)   = DMA_ENABLE + DMA_INTREN + DMA_STARTFIFO + \
			  DMA_SIZE32 + DMA_REPEAT + DMA_SRCINC + DMA_DSTFIX;
  wavecnt = wave_size;

  // Initialize timer

  cnt = 16777216 / wave_freq;		// Cycles = 2^24 / sampling freq
  register(TMR_COUNT0) = -cnt;		// Setup timer
  register(TMR_CTRL0) = TMR_ENABLE + TMR_1CLOCK;

  // Initialize key interruption

  register(KEY_INTCNT) = KEY_INTCND + KEY_INTEN + KEY_L + KEY_R;

  // Activate interruption!

  register(INT_IE) = INT_DMA1 | INT_KEY;
  register(INT_IME) = INT_IME_ON;

  // Loop forever

  selectpage(BG_TEXT1);
  locate(5, 8);
  printf("  Comment: %s", wave_comment);
  locate(5, 10);
  printf("Frequency: %d Hz", wave_freq);

  while (1) {
   }
 }
