//
// i n t p c m . c
//
// Interrupt-driven PCM player on GBA
//
// July 20, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "screen.h"
#include "console.h"
#include "printf.h"

// External global symbols

extern char wave_data[];
extern char wave_comment[];
extern int  wave_freq;
extern int  wave_size;

int* waveptr = (int*) wave_data;	// Wave data pointer
int  wavecnt = 0;			// Wave data counter

// Interrupt handler

void int_handler(void) {
  int flag;

  register(INT_IME) = INT_IME_OFF;	// Disable INTs
  flag = register(INT_IF);		// Read INT flags
  if (flag & INT_KEY) {			// This is keypad INT
    register(TMR_CTRL0) = 0;		// Disable timer
    waveptr = (int*) wave_data;		// Reset buffer pointer
    wavecnt = 0;			// Clear counter
    register4(SND_DAFIFO) = waveptr[0];	// Preload 1st data onto the FIFO
    register(TMR_CTRL0) =		// Activate timer
      TMR_ENABLE + TMR_1CLOCK + TMR_INTREN;
   }
  if (flag & INT_TMR0) {		// This is timer #0 INT
    if ((wavecnt % 4) == 0)		// Is this 1st data?
      register4(SND_DAFIFO) =		// Then, push 4 bytes to sound FIFO
        waveptr[ wavecnt >> 2 ];
    wavecnt++;				// Increment counter
    if (wavecnt > wave_size)		// End of job?
      register(TMR_CTRL0) = 0;		// Then disable timer
   }
  register(INT_IF) = flag;		// Reset INT flags
  register(INT_IME) = INT_IME_ON;	// Enable INTs
 }

int main() {
  int cnt;

  // Initialize screen

  init_screen();

  // Initialize sound controller and FIFO

  register(SND_MASTER)  = SND_ENABLE;
  register(SND_DOUTPUT) = SND_DARESET + SND_DAFULL +
                          SND_DALEFT + SND_DARIGHT;
  register4(SND_DAFIFO) = waveptr[ 0 ];	// Preload 1st data onto the FIFO

  // Initialize timer

  cnt = 16777216 / wave_freq;		// Cycles = 2^24 / sampling freq
  register(TMR_COUNT0) = 65536 - cnt;	// Setup timer
  register(TMR_CTRL0) = TMR_ENABLE + TMR_1CLOCK + TMR_INTREN;

  // Initialize key interruption

  register(KEY_INTCNT) = KEY_INTCND + KEY_INTEN + KEY_L + KEY_R;

  // Activate interruption!

  register(INT_IE) = INT_TMR0 | INT_KEY;
  register(INT_IME) = INT_IME_ON;

  // Loop forever

  selectpage(BG_TEXT1);
  locate(5, 8);
  printf("  Comment: %s", wave_comment);
  locate(5, 10);
  printf("Frequency: %d Hz", wave_freq);
  while (1) {
   }
 }
