//
// d m a s c r o l l . c
//
// Screen scroller based on DMA
//
// July 21, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "screen.h"
#include "console.h"
#include "printf.h"

// External "symbol" defined in linker script

extern hword image_start[];	// Graphic image data area

void int_handler(void) {
 }

void dma_xfer(hword* src, hword* dst, int words) {
  // Initialize and activate DMA controller

  register4(DMA_SRCAD3) = (int) src;
  register4(DMA_DSTAD3) = (int) dst;
  register(DMA_COUNT3)  = words;
  register(DMA_CTRL3)   = DMA_ENABLE + DMA_STARTIMD + DMA_SRCINC + DMA_DSTINC;
 }

int main() {
  int i;
  hword *sptr, *dptr;
  hword buf[ LCD_WIDTH ];
  unsigned short int tmr;

  // Initialize graphic screen

  register(LCD_CTRL) = LCD_BG2EN + LCD_MODE3;	// Mode 3
  sptr = image_start;		// Load screen image data onto VRAM
  dptr = (hword*) VRAM;
  dma_xfer(sptr, dptr, LCD_WIDTH * LCD_HEIGHT);

  // Setup timer #1 to msec counter

  register(TMR_COUNT0) = 0xBE77;		// Timer #0 overflows in 1msec
  register(TMR_CTRL0) = TMR_ENABLE + TMR_1CLOCK;
  register(TMR_COUNT1) = 0;			// Timer #1 is cascaded
  register(TMR_CTRL1) = TMR_ENABLE + TMR_CASCADE;

  // Scroll up the screen

  for (i = 0; i < LCD_HEIGHT; i++) {
    sptr = (hword*) VRAM;			// Save top line
    dptr = (hword*) buf;
    dma_xfer(sptr, dptr, LCD_WIDTH);

    sptr = (hword*) (VRAM + (LCD_WIDTH * 2));	// Scroll up one line
    dptr = (hword*) VRAM;
    dma_xfer(sptr, dptr, LCD_WIDTH * (LCD_HEIGHT-1));

    sptr = (hword*) buf;			// Move previous top to bottom
    dptr = (hword*) (VRAM + LCD_WIDTH * (LCD_HEIGHT - 1) * 2);
    dma_xfer(sptr, dptr, LCD_WIDTH);
   }
  tmr = register(TMR_COUNT1);

  // Initialize text screen

  register(LCD_CTRL) = 0;			// Blank the screen
  init_screen();				// Initialize screen module
  selectpage(BG_TEXT1);
  locate(2, 10);
  printf("Elasped time is %d msec\n", tmr);
  return 0;
 }
