//
// m o v i n g . c
//
// Moving ball...
//
// July 27, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "screen.h"
#include "console.h"
#include "printf.h"
#include "sprite.h"

// External symbols

extern char wave_data[];
extern char wave_comment[];
extern int  wave_freq;
extern int  wave_size;

// Global symbols

int wavecnt = 0;			// Wave data counter
int vblank = 0;				// Vertical blank counter
int x = 118, y = 72;			// Ball position
int ax = 3, dx = 3, ay = 3, dy = 3;	// Ball controller

// Sound starter

void trigger_sound(void) {
  register(DMA_CTRL1)   = 0;		// Disable DMA
  register(TMR_CTRL0)   = 0;		// Disable timer
  register4(DMA_DSTAD1) = SND_DAFIFO;	// Reset DMA controller
  register4(DMA_SRCAD1) = (int) wave_data;
  register(DMA_CTRL1)   = DMA_ENABLE + DMA_INTREN + DMA_STARTFIFO + \
			  DMA_SIZE32 + DMA_REPEAT + DMA_SRCINC + DMA_DSTFIX;
  wavecnt = wave_size;			// Reset counter
  register(TMR_CTRL0) =			// Activate timer
      TMR_ENABLE + TMR_1CLOCK;
 }

// Interrupt handler

void int_handler(void) {
  int flag;

  register(INT_IME) = INT_IME_OFF;	// Disable INTs
  flag = register(INT_IF);		// Read INT flag bits
  if (flag & INT_KEY) {			// This is keypad INT
    if ((register(KEY_STATUS) & KEY_UP) == 0)
      ay = (ay < 9) ? ay + 1 : 9;
    if ((register(KEY_STATUS) & KEY_DOWN) == 0)
      ay = (ay > 1) ? ay - 1 : 1;
    if ((register(KEY_STATUS) & KEY_RIGHT) == 0)
      ax = (ax < 9) ? ax + 1 : 9;
    if ((register(KEY_STATUS) & KEY_LEFT) == 0)
      ax = (ax > 1) ? ax - 1 : 1;
    dx = (dx < 0) ? -ax : ax;
    dy = (dy < 0) ? -ay : ay;
   }
  if (flag & INT_DMA1) {		// This is DMA1 INT
    wavecnt -= 16;			// Decrement counter
    if (wavecnt <= 0) {			// End of job?
      register(TMR_CTRL0) = 0;		// Then disable timer 0
      register(DMA_CTRL1) = 0;		// and DMA 1
     }
   }
  if (flag & INT_VBLANK)		// This is vertical blanking INT
    vblank++;				// Increment counter
  register(INT_IF) = flag;		// Reset INT flags
  register(INT_IME) = INT_IME_ON;	// Enable INTs
 }

int main() {
  int cnt, cnum;
  int snd;
  extern character greenball[];

  // Initialize screen

  init_screen();

  // Initialize sprite

  init_sprite();
  cnum = sprite_register(greenball, 4);
  sprite_setup(0, OBJ_16x16, PRIORITY_2ND, cnum);
  sprite_display(0, OBJ_TURNON);

  // Initialize sound controller

  register(SND_MASTER)  = SND_ENABLE;
  register(SND_DOUTPUT) = SND_DARESET + SND_DAFULL +
                          SND_DALEFT + SND_DARIGHT;
  // Initialize timer

  cnt = 16777216 / wave_freq;		// Cycles = 2^24 / sampling freq
  register(TMR_COUNT0) = -cnt;		// Setup timer
  register(TMR_CTRL0) = TMR_ENABLE + TMR_1CLOCK;

  // Initialize keypad INT

  register(KEY_INTCNT) = KEY_INTEN + KEY_UP + KEY_DOWN + KEY_LEFT + KEY_RIGHT;

  // Activate interruption!

  register(LCD_STATUS) |= LCD_VBINTEN;
  register(INT_IE) = INT_DMA1 | INT_KEY | INT_VBLANK;
  register(INT_IME) = INT_IME_ON;

  // Loop forever

  selectpage(BG_TEXT1);
  locate(7, 9);
  printf("%c Moving ball! %c", 13, 14);
  while (1) {
    locate(9, 12);
    printf("vblank = %d", vblank);
    locate(9, 14);
    printf("ax=%d : ", ax);
    printf("ay=%d", ay);
    cnt = vblank;
    while (cnt == vblank);
    x += dx; y += dy;
    snd = 0;
    if (x >= (LCD_WIDTH - 16)) {
      trigger_sound(); snd = 1;
      dx = -ax;
     }
    else if (x < 0) {
      trigger_sound(); snd = 1;
      dx = ax;
     }
    if (y >= (LCD_HEIGHT - 16)) {
      if (!snd) trigger_sound();
      dy = -ay;
     }
    else if (y < 0) {
      if (!snd) trigger_sound();
      dy = ay;
     }
    sprite_move(0, x, y);
   }
 }
