//
// s c r e e n . c
//
// LCD screen driver
//
// June 22, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "8x8.til"	// 8x8 font data

// Screen information

screen scrn[ 3 ] = {
  { (hword*) VRAM_SCRN(21), (hword*) VRAM_CHAR(3), 0, 0 },	// Page 0
  { (hword*) VRAM_SCRN(22), (hword*) VRAM_CHAR(3), 0, 0 },	// Page 1
  { (hword*) VRAM_SCRN(23), (hword*) VRAM_CHAR(0), 0, 0 },	// Page 2
 };

int    page = 1;			// Page number
hword* bcolor = (hword*) BG_PALETTE;	// 0x05000000

// External background data

extern unsigned short int bg_tiles[][32];
extern unsigned short int bg_palette[];

void init_screen(void) {
  int i, j, row, col, bit, val;

  // Initialize palettes

  for (i = 0; i < LCD_PAL_COLORS; i++)
    bcolor[ i ] = bg_palette[ i ];

  // Setup tiles for ASCII codes

  for (i = 0; i < 256; i++) {
    for (row = 0; row < 8; row++) {
      for (col = 7; col >= 0; col--) {
        bit = (char8x8[ i ][ row ] & (1 << col)) ? BG_WHITE : BG_BLACK;
        if (col % 2)
          val = bit;
        else
          scrn[ BG_TEXT1 ].tile[(i*LCD_TILE_HWORD)+(row*4)+((7-col)/2)] = \
          val + (bit << 8);
       }
     }
   }

  // Setup tiles for background

  for (i = 0; i < (LCD_RWIDTH * LCD_RHEIGHT + 1); i++) {
    for (j = 0; j < LCD_TILE_HWORD; j++) {
      scrn[ BG_GRAPHIC ].tile[ (i * LCD_TILE_HWORD) + j ] = bg_tiles[ i ][ j ];
     }
   }

  // Clear text screens

  for (i = 0; i < LCD_RHEIGHT; i++) {
    for (j = 0; j < LCD_RWIDTH; j++) {
      scrn[ BG_TEXT1 ].map[ i * LCD_VWIDTH + j ] = 0x20;
      scrn[ BG_TEXT2 ].map[ i * LCD_VWIDTH + j ] = 0x20;
     }
   }

  // Arrange the background

  for (i = 0; i < LCD_RHEIGHT; i++)
    for (j = 0; j < LCD_RWIDTH; j++)
      scrn[ BG_GRAPHIC ].map[ i * LCD_VWIDTH + j ] = (i * LCD_RWIDTH) + j;

  // Start LCD display

  register(LCD_BG0)  = LCD_SIZE3232  | LCD_COLOR256 |
                       LCD_BGCHAR(3) | LCD_BGSCRN(21) | PRIORITY_1ST;
  register(LCD_BG1)  = LCD_SIZE3232  | LCD_COLOR256 |
                       LCD_BGCHAR(3) | LCD_BGSCRN(22) | PRIORITY_2ND;
  register(LCD_BG2)  = LCD_SIZE3232  | LCD_COLOR256 |
                       LCD_BGCHAR(0) | LCD_BGSCRN(23) | PRIORITY_3RD;
  register(LCD_CTRL) = LCD_BG0EN | LCD_BG1EN | LCD_BG2EN | LCD_MODE0;
 }
