//
// m a k e g b a . c
//
// Create a GBA sound file
//
// June 25, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include <stdlib.h>	// atoi(), atof()
#include <math.h>	// sin()
#include <stdio.h>	// printf()

// Constant definitions

#define MAXAMP	128	// Maximum WAVE amplitude
#define PI	3.14159	// The circular constant

// Global variables

int sampling;		// Sampling frequency (Hz)
int seconds;		// Sound length (seconds)

int main(int argc, char* argv[]) {
  int i;
  double freq1 = 0, freq2 = 0, freq3 = 0, freq4 = 0, freq5 = 0, amp;
  double data;

  if (argc < 5 || argc > 9) {
    printf("makegba sampling_rate \"comment\" sec freq1 [ freq2 [ freq3 [ freq4 [ freq5 ]]]]\n");
    return 1;
   }

  // Read frequencies
  sampling = atoi(argv[ 1 ]);
  seconds = atoi(argv[ 3 ]);
  freq1 = atof(argv[ 4 ]);
  if (argc >= 6)
    freq2 = atof(argv[ 5 ]);
  if (argc >= 7)
    freq3 = atof(argv[ 6 ]);
  if (argc >= 8)
    freq4 = atof(argv[ 7 ]);
  if (argc == 9)
    freq5 = atof(argv[ 8 ]);

  // Adjust wave amplitude

  argc -= 4;
  switch (argc) {
    case 1:
      amp = MAXAMP;
      break;

    case 2:
      amp = MAXAMP * 0.55;
      break;

    case 3:
      amp = MAXAMP * 0.3;
      break;

    case 4:
      amp = MAXAMP * 0.25;
      break;

    case 5:
      amp = MAXAMP * 0.2;
      break;
   }

  printf("// GBA sound file\n");
  printf("// Sampling rate = %d Hz : Time = %d sec\n", sampling, seconds);
  printf("// Freq1 = %f\n// Freq2 = %f\n// Freq3 = %f\n", freq1, freq2, freq3);
  printf("// Freq4 = %f\n// Freq5 = %f\n\n", freq4, freq5);
  printf("const char wave_comment[] = \"%s\";\n", argv[ 2 ]);
  printf("int wave_freq = %d;\n", sampling);
  printf("int wave_size = %d;\n\n", sampling * seconds);
  printf("char wave_data[] = {\n  ");
  for (i = 0; i < sampling * seconds; i++) {
    data = amp * sin(2 * PI * freq1 / sampling * i);
    if (freq2)
      data += amp * sin(2 * PI * freq2 / sampling * i);
    if (freq3)
      data += amp * sin(2 * PI * freq3 / sampling * i);
    if (freq4)
      data += amp * sin(2 * PI * freq4 / sampling * i);
    if (freq5)
      data += amp * sin(2 * PI * freq5 / sampling * i);
    printf("%d, ", (int) data);
    if ((i % 10) == 9) printf("\n  ");
   }
  printf(" };\n");
  return 0;
 }
